using DAL;
using DAL.Models;
using Infrastructure.EFCore.Query;
using Microsoft.EntityFrameworkCore;

namespace Infrastructure.EFCore.Test
{
    public class QueryTests
    {
        private readonly SchoolDBContext dbContext;

        public QueryTests()
        {
            var myDatabaseName = "mydatabase_" + DateTime.Now.ToFileTimeUtc();

            var dbContextOptions = new DbContextOptionsBuilder<SchoolDBContext>()
                .UseInMemoryDatabase(databaseName: myDatabaseName)
                .Options;

            dbContext = new SchoolDBContext(dbContextOptions);

            dbContext.Classrooms.Add(new Classroom { Id = 1, Name = "B113" });
            dbContext.Classrooms.Add(new Classroom { Id = 2, Name = "B117" });
            dbContext.Classrooms.Add(new Classroom { Id = 3, Name = "A217" });

            dbContext.SaveChanges();
        }

        [Fact]
        public void ClassroomA217Exists_QueryWhere_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.Where<string>(a => a == "B113", "Name");
            var result = efquery.Execute();

            Assert.True(result.Count() == 1);

            Assert.True(result.First().Name == "B113");
        }

        [Fact]
        public void ClassroomsThatStartWithBExists_QueryWhere_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.Where<string>(a => a.StartsWith("B"), "Name");
            var result = efquery.Execute();

            Assert.True(result.Count() == 2);
        }

        [Fact]
        public void ClassroomsWithIDLessThan2_QueryWhere_Test()
        {
            var efquery = new EntityFrameworkQuery<Classroom>(dbContext);
            efquery.Where<int>(a => a < 3, "Id");
            var result = efquery.Execute();

            Assert.True(result.Count() == 2);
        }
    }
}