﻿using DAL;
using DAL.Models;
using Infrastructure.EFCore.ExpressionHelpers;
using Infrastructure.EFCore.UnitOfWork;
using Infrastructure.Query;
using Infrastructure.UnitOfWork;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Infrastructure.EFCore.Query
{
    public class EntityFrameworkQuery<TEntity> : Query<TEntity> where TEntity : class, IEntity, new()
    {
        protected SchoolDBContext Dbcontext { get; set; }

        private EFUnitOfWork _unitOfWork;

        protected EFUnitOfWork UnitOfWork 
        { 
            get 
            { 
                if (_unitOfWork != null)
                {
                    _unitOfWork = new(Dbcontext);
                }

                return _unitOfWork;
            } 
        }

        public EntityFrameworkQuery(SchoolDBContext dbcontext)
        {
            Dbcontext = dbcontext;
        }

        public override IEnumerable<TEntity> Execute()
        {
            IQueryable<TEntity> query = Dbcontext.Set<TEntity>();

            if (WherePredicate.Capacity != 0)
            {
                query = ApplyWhere(query);
            }

            return query.ToList();

        }

        private IQueryable<TEntity> ApplyWhere(IQueryable<TEntity> query)
        {
            foreach (var expr in WherePredicate)
            {
                // parameter for new lambda
                var p = Expression.Parameter(typeof(TEntity), "p");

                // get the property name from the etity. For instance, its the same as calling `nameof(Subject.Name)`
                var columnNameFromObject = typeof(TEntity)
                    .GetProperty(expr.columnName)
                    ?.Name;

                // basically creates the property call, i.e -> p.Name
                var exprProp = Expression.Property(p, columnNameFromObject);// nameof(Customer.CustomerID));
                                                                            // replace parameter in original expression
                var expression = expr.expression;

                // gets the Expression Parameters
                var parameters = (IReadOnlyCollection<ParameterExpression>)expression
                    .GetType()
                    .GetProperty("Parameters")
                    ?.GetValue(expression);

                // gets the expression body
                var body = (Expression)expression
                    .GetType()
                    .GetProperty("Body")
                    ?.GetValue(expression);

                // (both) replaces the old lambda parameter with the new one
                // Example:
                //      a => a > 10
                // ->   p => p.Price > 10
                var visitor = new ReplaceParamVisitor(parameters.First(), exprProp);
                var exprNewBody = visitor.Visit(body);

                // creates the new lambda expression
                var lambda = Expression.Lambda<Func<TEntity, bool>>(exprNewBody, p);

                query = query.Where(lambda);
            }

            return query;
        }
    }
}
