﻿using DAL;
using DAL.Models;
using Infrastructure.EFCore.Repository;
using Infrastructure.Repository;
using Infrastructure.UnitOfWork;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Infrastructure.EFCore.UnitOfWork
{
    public class EFUnitOfWork : IUnitOfWork
    {
        public SchoolDBContext Context { get; } = new();
        private IRepository<Student> studentRepository;
        private IRepository<Classroom> classroomRepository;
        private IRepository<Subject> subjectRepository;

        public EFUnitOfWork(SchoolDBContext dbContext)
        {
            Context = dbContext;
        }

        public IRepository<Student> StudentRepository
        {
            get
            {
                if (this.studentRepository == null)
                {
                    this.studentRepository = new EFGenericRepository<Student>(Context);
                }
                return studentRepository;
            }
        }

        public IRepository<Classroom> ClassroomRepository
        {
            get
            {
                if (this.subjectRepository == null)
                {
                    this.classroomRepository = new EFGenericRepository<Classroom>(Context);
                }
                return classroomRepository;
            }
        }

        public IRepository<Subject> SubjectRepository
        {
            get
            {
                if (this.subjectRepository == null)
                {
                    this.subjectRepository = new EFGenericRepository<Subject>(Context);
                }
                return subjectRepository;
            }
        }

        public async Task Commit()
        {
            await Context.SaveChangesAsync();
        }

        public void Dispose()
        {
            Context.Dispose();
        }
    }
}
